@@ -19,14 +19,11 @@ private:
     unsigned long countNumChildren(KSysGuard::Process *p);
 private slots:
     void testTimeToUpdateAllProcesses();
-    void testTimeToUpdateModel();
     void testProcesses();
     void testProcessesTreeStructure();
     void testProcessesModification();
     void testHistories();
-    void testHistoriesWithWidget();
     void testUpdateOrAddProcess();
-    void testCPUGraphHistory();
 
     void testSetScheduler();
     void testSetScheduler_data();
