@@ -12,13 +12,8 @@
 
 #include <QObject>
 
-#include <kauth_version.h>
-#if KAUTH_VERSION >= QT_VERSION_CHECK(5, 92, 0)
 #include <KAuth/ActionReply>
 #include <KAuth/HelperSupport>
-#else
-#include <kauth.h>
-#endif
 
 using namespace KAuth;
 
