@@ -5,8 +5,9 @@
 */
 
 #include "process.h"
+
+#include "memoryinfo_p.h"
 #include "processes_local_p.h"
-#include "read_procsmaps_runnable.h"
 
 #include <klocalizedstring.h>
 
@@ -14,7 +15,7 @@
 #include <QDir>
 #include <QFile>
 #include <QHash>
-#include <QPointer>
+#include <QQueue>
 #include <QSet>
 #include <QTextStream>
 #include <QThreadPool>
@@ -34,6 +35,8 @@
 // for getsched
 #include <sched.h>
 
+#include <thread>
+
 #define PROCESS_BUFFER_SIZE 1000
 
 /* For ionice */
@@ -86,6 +89,15 @@ static int ioprio_get(int which, int who)
 }
 #endif
 
+// The rate at which the thread reading /proc/PID/smaps_rollup reads batches.
+constexpr auto SmapsThreadDelay = std::chrono::milliseconds(100);
+// The number of PIDs to read in a single batch.
+constexpr auto SmapsThreadBatchSize = 5;
+// The minimum time between updates of precise memory data.
+constexpr auto PreciseUpdateInterval = std::chrono::seconds(30);
+
+using namespace Qt::StringLiterals;
+
 namespace KSysGuard
 {
 class ProcessesLocal::Private
@@ -93,6 +105,7 @@ class ProcessesLocal::Private
 public:
     Private()
     {
+        mBuffer.fill('\0', PROCESS_BUFFER_SIZE);
         mProcDir = opendir("/proc");
     }
     ~Private();
@@ -104,21 +117,82 @@ public:
     inline bool readProcAttr(const QString &dir, Process *process);
     inline bool getNiceness(long pid, Process *process);
     inline bool getIOStatistics(const QString &dir, Process *process);
+
+    static void smapsThreadFunction(std::stop_token stopToken, ProcessesLocal *processes);
+
     QFile mFile;
-    char mBuffer[PROCESS_BUFFER_SIZE + 1]; // used as a buffer to read data into
+    QByteArray mBuffer;
     DIR *mProcDir;
-    QPointer<ReadProcSmapsRunnable> smapsRunnable;
+
+    std::unique_ptr<std::jthread> smapsThread = nullptr;
+    QQueue<long> smapsQueue;
+    std::mutex smapsQueueMutex;
 };
 
 ProcessesLocal::Private::~Private()
 {
+    if (smapsThread) {
+        smapsThread->request_stop();
+        smapsThread->join();
+    }
     closedir(mProcDir);
 }
 
+void ProcessesLocal::Private::smapsThreadFunction(std::stop_token stopToken, ProcessesLocal *processes)
+{
+    while (!stopToken.stop_requested()) {
+        std::this_thread::sleep_for(SmapsThreadDelay);
+
+        std::vector<long> pids;
+        {
+            std::lock_guard<std::mutex> lock(processes->d->smapsQueueMutex);
+            while (!processes->d->smapsQueue.isEmpty() && pids.size() < SmapsThreadBatchSize) {
+                pids.push_back(processes->d->smapsQueue.takeFirst());
+            }
+        }
+
+        if (pids.empty()) {
+            continue;
+        }
+
+        for (auto pid : pids) {
+            QFile file{"/proc/"_L1 + QString::number(pid) + "/smaps_rollup"_L1};
+            if (!file.open(QIODevice::ReadOnly)) {
+                continue;
+            }
+
+            MemoryFields fields;
+
+            auto buffer = QByteArray{1024, '\0'};
+            while (file.readLine(buffer.data(), buffer.size()) > 0) {
+                auto parts = buffer.split(':');
+                if (parts.size() >= 2) {
+                    if (parts.at(0) == "Rss") {
+                        fields.rss += std::stoll(parts.at(1).toStdString());
+                    } else if (parts.at(0) == "Pss") {
+                        fields.pss += std::stoll(parts.at(1).toStdString());
+                    } else if (parts.at(0).startsWith("Shared")) {
+                        fields.shared += std::stoll(parts.at(1).toStdString());
+                    } else if (parts.at(0).startsWith("Private")) {
+                        fields.priv += std::stoll(parts.at(1).toStdString());
+                    }
+                }
+            }
+
+            file.close();
+
+            fields.lastUpdate = std::chrono::steady_clock::now();
+
+            Q_EMIT processes->processUpdated(pid, {{Process::MemoryPrecise, QVariant::fromValue(fields)}});
+        }
+    }
+}
+
 ProcessesLocal::ProcessesLocal()
     : d(new Private())
 {
 }
+
 bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *process)
 {
     mFile.setFileName(dir + QStringLiteral("status"));
@@ -133,7 +207,7 @@ bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *proces
 
     int size;
     int found = 0; // count how many fields we found
-    while ((size = mFile.readLine(mBuffer, sizeof(mBuffer))) > 0) { //-1 indicates an error
+    while ((size = mFile.readLine(mBuffer.data(), mBuffer.size())) > 0) { //-1 indicates an error
         switch (mBuffer[0]) {
         case 'N':
             if ((unsigned int)size > sizeof("Name:") && qstrncmp(mBuffer, "Name:", sizeof("Name:") - 1) == 0) {
@@ -195,6 +269,10 @@ bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *proces
                 }
             }
             break;
+        case 'V':
+            if (mBuffer.startsWith("VmSwap:")) {
+                process->memoryInfo()->imprecise.swap = atol(mBuffer + sizeof("VmSwap:") - 1);
+            }
         default:
             break;
         }
@@ -212,7 +290,7 @@ bool ProcessesLocal::Private::readProcCGroup(const QString &dir, Process *proces
         return false; /* process has terminated in the meantime */
     }
 
-    while (mFile.readLine(mBuffer, sizeof(mBuffer)) > 0) { //-1 indicates an error
+    while (mFile.readLine(mBuffer.data(), mBuffer.size()) > 0) { //-1 indicates an error
         if (mBuffer[0] == '0' && mBuffer[1] == ':' && mBuffer[2] == ':') {
             process->setCGroup(QString::fromLocal8Bit(&mBuffer[3]).trimmed());
             break;
@@ -229,7 +307,7 @@ bool ProcessesLocal::Private::readProcAttr(const QString &dir, Process *process)
         return false; /* process has terminated in the meantime */
     }
 
-    if (mFile.readLine(mBuffer, sizeof(mBuffer)) > 0) { //-1 indicates an error
+    if (mFile.readLine(mBuffer.data(), mBuffer.size()) > 0) { //-1 indicates an error
         process->setMACContext(QString::fromLocal8Bit(mBuffer).trimmed());
     }
     mFile.close();
@@ -247,13 +325,13 @@ long ProcessesLocal::getParentPid(long pid)
     }
 
     int size; // amount of data read in
-    if ((size = d->mFile.readLine(d->mBuffer, sizeof(d->mBuffer))) <= 0) { //-1 indicates nothing read
+    if ((size = d->mFile.readLine(d->mBuffer.data(), d->mBuffer.size())) <= 0) { //-1 indicates nothing read
         d->mFile.close();
         return -1;
     }
 
     d->mFile.close();
-    char *word = d->mBuffer;
+    char *word = d->mBuffer.data();
     // The command name is the second parameter, and this ends with a closing bracket.  So find the last
     // closing bracket and start from there
     word = strrchr(word, ')');
@@ -289,14 +367,14 @@ bool ProcessesLocal::Private::readProcStat(const QString &dir, Process *ps)
         if (!mFile.open(QIODevice::ReadOnly)) {
             return false; /* process has terminated in the meantime */
         }
-        if (mFile.readLine(mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
+        if (mFile.readLine(mBuffer.data(), mBuffer.size()) <= 0) { //-1 indicates nothing read
             mFile.close();
             return false;
         }
         mFile.close();
     }
 
-    char *word = mBuffer;
+    char *word = mBuffer.data();
     // The command name is the second parameter, and this ends with a closing bracket.  So find the last
     // closing bracket and start from there
     word = strrchr(word, ')');
@@ -306,8 +384,6 @@ bool ProcessesLocal::Private::readProcStat(const QString &dir, Process *ps)
     word++; // Nove to the space after the last ")"
     int current_word = 1; // We've skipped the process ID and now at the end of the command name
     char status = '\0';
-    unsigned long long vmSize = 0;
-    unsigned long long vmRSS = 0;
     while (current_word < 23) {
         if (word[0] == ' ') {
             ++current_word;
@@ -352,10 +428,10 @@ bool ProcessesLocal::Private::readProcStat(const QString &dir, Process *ps)
                 ps->setStartTime(atoll(word + 1));
                 break;
             case 22: // vmSize
-                vmSize = atoll(word + 1);
+                // Does nothing, read from statm below.
                 break;
             case 23: // vmRSS
-                vmRSS = atoll(word + 1);
+                // Does nothing, read from statm below.
                 break;
             default:
                 break;
@@ -366,16 +442,6 @@ bool ProcessesLocal::Private::readProcStat(const QString &dir, Process *ps)
         word++;
     }
 
-    /* There was a "(ps->vmRss+3) * sysconf(_SC_PAGESIZE)" here in the original ksysguard code.  I have no idea why!  After comparing it to
-     *   meminfo and other tools, this means we report the RSS by 12 bytes differently compared to them.  So I'm removing the +3
-     *   to be consistent.  NEXT TIME COMMENT STRANGE THINGS LIKE THAT! :-)
-     *
-     *   Update: I think I now know why - the kernel allocates 3 pages for
-     *   tracking information about each the process. This memory isn't
-     *   included in vmRSS..*/
-    ps->setVmRSS(vmRSS * (sysconf(_SC_PAGESIZE) / 1024)); /*convert to KiB*/
-    ps->setVmSize(vmSize / 1024); /* convert to KiB */
-
     switch (status) {
     case 'R':
         ps->setStatus(Process::Running);
@@ -410,34 +476,51 @@ bool ProcessesLocal::Private::readProcStatm(const QString &dir, Process *process
         return false; /* process has terminated in the meantime */
     }
 
-    if (mFile.readLine(mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
+    qint64 size = 0;
+    if (size = mFile.readLine(mBuffer.data(), mBuffer.size()); size <= 0) { //-1 indicates nothing read
         mFile.close();
-        return 0;
+        return false;
     }
     mFile.close();
 
-    int current_word = 0;
-    char *word = mBuffer;
+    MemoryFields fields;
+    auto parts = mBuffer.mid(0, size).split(' ');
 
-    while (true) {
-        if (word[0] == ' ') {
-            if (++current_word == 2) {
-                // number of pages that are shared
-                break;
-            }
-        } else if (word[0] == 0) {
-            return false; // end of data - serious problem
-        }
-        word++;
+    if (parts.size() != 7) {
+        qWarning() << "Reading statm from" << dir + u"statm"_s << "failed, expected 7 fields, got" << parts.size();
+        return false;
     }
-    long shared = atol(word + 1);
 
-    /* we use the rss - shared  to find the amount of memory just this app uses */
-    process->setVmURSS(process->vmRSS() - (shared * sysconf(_SC_PAGESIZE) / 1024));
-#else
-    process->setVmURSS(0);
-#endif
+    static const auto pageSize = sysconf(_SC_PAGESIZE) / 1024;
+
+    // From kernel documentation:
+    // Table 1-3: Contents of the statm files (as of 2.6.8-rc3)
+    // ..............................................................................
+    // Field    Content
+    // size     total program size (pages)		(same as VmSize in status)
+    // resident size of memory portions (pages)	(same as VmRSS in status)
+    // shared   number of pages that are shared	(i.e. backed by a file, same
+    //                                              as RssFile+RssShmem in status)
+    // trs      number of pages that are 'code'	(not including libs; broken,
+    //                                              includes data segment)
+    // lrs      number of pages of library		(always 0 on 2.6)
+    // drs      number of pages of data/stack	(including libs; broken,
+    //                                              includes library text)
+    // dt       number of dirty pages			(always 0 on 2.6)
+    fields.rss = atol(parts.at(1)) * pageSize;
+    fields.shared = atol(parts.at(2)) * pageSize;
+    fields.priv = fields.rss - fields.shared;
+    fields.swap = process->swap();
+    fields.lastUpdate = std::chrono::steady_clock::now();
+
+    process->memoryInfo()->imprecise = fields;
+    process->memoryInfo()->vmSize = atol(parts.at(0)) * pageSize;
+    process->addChange(Process::Memory);
+
     return true;
+#else
+    return false;
+#endif
 }
 
 bool ProcessesLocal::Private::readProcCmdline(const QString &dir, Process *process)
@@ -533,14 +616,14 @@ bool ProcessesLocal::Private::getIOStatistics(const QString &dir, Process *proce
     if (!mFile.open(QIODevice::ReadOnly)) {
         return false; /* process has terminated in the meantime */
     }
-    if (mFile.read(mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
+    if (mFile.readLine(mBuffer.data(), mBuffer.size()) <= 0) { //-1 indicates nothing read
         mFile.close();
         return false;
     }
     mFile.close();
 
     int current_word = 0; // count from 0
-    char *word = mBuffer;
+    char *word = mBuffer.data();
     while (current_word < 6 && word[0] != 0) {
         if (word[0] == ' ') {
             qlonglong number = atoll(word + 1);
@@ -576,14 +659,17 @@ bool ProcessesLocal::updateProcessInfo(long pid, Process *process)
     bool success = true;
     const QString dir = QLatin1String("/proc/") + QString::number(pid) + QLatin1Char('/');
 
-    if (mUpdateFlags.testFlag(Processes::Smaps) && !d->smapsRunnable) {
-        d->smapsRunnable = new ReadProcSmapsRunnable{dir};
+    if (mUpdateFlags.testFlag(Processes::Smaps)) {
+        if (!d->smapsThread) {
+            d->smapsThread = std::make_unique<std::jthread>(Private::smapsThreadFunction, this);
+        }
 
-        connect(d->smapsRunnable, &ReadProcSmapsRunnable::finished, this, [this, pid](qulonglong pss) {
-            Q_EMIT processUpdated(pid, {{Process::VmPSS, pss}});
-        });
-
-        QThreadPool::globalInstance()->start(d->smapsRunnable);
+        if (std::chrono::steady_clock::now() - process->memoryInfo()->precise.lastUpdate >= PreciseUpdateInterval) {
+            std::lock_guard<std::mutex> lock(d->smapsQueueMutex);
+            if (!d->smapsQueue.contains(pid)) {
+                d->smapsQueue.append(pid);
+            }
+        }
     }
 
     if (!d->readProcStat(dir, process)) {
@@ -802,7 +888,7 @@ long long ProcessesLocal::totalPhysicalMemory()
     }
 
     int size;
-    while ((size = d->mFile.readLine(d->mBuffer, sizeof(d->mBuffer))) > 0) { //-1 indicates an error
+    while ((size = d->mFile.readLine(d->mBuffer.data(), d->mBuffer.size())) > 0) { //-1 indicates an error
         switch (d->mBuffer[0]) {
         case 'M':
             if ((unsigned int)size > sizeof("MemTotal:") && qstrncmp(d->mBuffer, "MemTotal:", sizeof("MemTotal:") - 1) == 0) {
@@ -814,9 +900,27 @@ long long ProcessesLocal::totalPhysicalMemory()
     return 0; // Not found.  Probably will never happen
 #endif
 }
+
+long long ProcessesLocal::totalSwapMemory()
+{
+    d->mFile.setFileName("/proc/meminfo");
+    if (!d->mFile.open(QIODevice::ReadOnly)) {
+        return 0;
+    }
+
+    int size;
+    while ((size = d->mFile.readLine(d->mBuffer.data(), d->mBuffer.size())) > 0) {
+        if (d->mBuffer.startsWith("SwapTotal:")) {
+            d->mFile.close();
+            return std::atoll(d->mBuffer.mid(sizeof("SwapTotal:") + 1));
+        }
+    }
+
+    return 0;
+}
+
 ProcessesLocal::~ProcessesLocal()
 {
     delete d;
 }
-
 }
