@@ -26,8 +26,8 @@ QVariant addVariants(const QVariant &first, const QVariant &second)
     bool convertFirst = false;
     bool convertSecond = false;
 
-    auto type = first.type();
-    switch (static_cast<QMetaType::Type>(type)) {
+    auto type = first.typeId();
+    switch (type) {
     case QMetaType::Char:
     case QMetaType::Short:
     case QMetaType::Int:
@@ -54,7 +54,7 @@ QVariant addVariants(const QVariant &first, const QVariant &second)
         return first;
     }
 
-    if (!result.convert(type)) {
+    if (!result.convert(QMetaType(type))) {
         return first;
     }
 
@@ -71,7 +71,8 @@ public:
     int dataCompressionDuration = 100;
     SensorContainer *subsystem = nullptr;
 
-    std::function<QVariant(AggregateSensor::SensorIterator, const AggregateSensor::SensorIterator)> aggregateFunction;
+    AggregateFunction aggregateFunction;
+    FilterFunction filterFunction;
 };
 
 QVariant AggregateSensor::SensorIterator::operator*() const
@@ -117,6 +118,10 @@ AggregateSensor::AggregateSensor(SensorObject *provider, const QString &id, cons
     setAggregateFunction(addVariants);
     connect(d->subsystem, &SensorContainer::objectAdded, this, &AggregateSensor::updateSensors);
     connect(d->subsystem, &SensorContainer::objectRemoved, this, &AggregateSensor::updateSensors);
+
+    d->filterFunction = [](const SensorProperty *) {
+        return true;
+    };
 }
 
 AggregateSensor::~AggregateSensor() = default;
@@ -185,7 +190,7 @@ void AggregateSensor::setMatchSensors(const QRegularExpression &objectIds, const
     updateSensors();
 }
 
-std::function<QVariant(AggregateSensor::SensorIterator, AggregateSensor::SensorIterator)> AggregateSensor::aggregateFunction() const
+AggregateSensor::AggregateFunction AggregateSensor::aggregateFunction() const
 {
     return d->aggregateFunction;
 }
@@ -205,17 +210,32 @@ void AggregateSensor::setAggregateFunction(const std::function<QVariant(QVariant
     d->aggregateFunction = aggregateFunction;
 }
 
-void AggregateSensor::setAggregateFunction(const std::function<QVariant(SensorIterator, const SensorIterator)> &newAggregateFunction)
+void AggregateSensor::setAggregateFunction(const AggregateFunction &newAggregateFunction)
 {
     d->aggregateFunction = newAggregateFunction;
 }
 
+AggregateSensor::FilterFunction AggregateSensor::filterFunction() const
+{
+    return d->filterFunction;
+}
+
+void AggregateSensor::setFilterFunction(const FilterFunction &function)
+{
+    d->filterFunction = function;
+    updateSensors();
+}
+
 void AggregateSensor::addSensor(SensorProperty *sensor)
 {
     if (!sensor || sensor->path() == path() || d->sensors.contains(sensor->path())) {
         return;
     }
 
+    if (!d->filterFunction(sensor)) {
+        return;
+    }
+
     if (isSubscribed()) {
         sensor->subscribe();
     }
@@ -256,6 +276,12 @@ void AggregateSensor::updateSensors()
                 itr.value()->unsubscribe();
             }
             itr = d->sensors.erase(itr);
+        } else if (!d->filterFunction(itr.value())) {
+            itr.value()->disconnect(this);
+            if (isSubscribed()) {
+                itr.value()->unsubscribe();
+            }
+            itr = d->sensors.erase(itr);
         } else {
             ++itr;
         }
